@@ -30,21 +30,16 @@ setGeneric("lu", function(x, ...) standardGeneric("lu"))
 
     setGeneric("Schur", function(x, vectors, ...) standardGeneric("Schur"))
 
-    setGeneric("unpack", function(x, ...) standardGeneric("unpack"))
+setGeneric("unpack", function(x, ...) standardGeneric("unpack"))
+setGeneric("pack", function(x, ...) standardGeneric("pack"))
 
 ##-     setGeneric("%p%", function(a, b) standardGeneric("%p%"))
 
     setGeneric("expm", function(x) standardGeneric("expm"))
 
-##     setGeneric("writeHB", function(obj, file, ...)
-##                standardGeneric("writeHB"))
-
     setGeneric("writeMM", function(obj, file, ...)
                standardGeneric("writeMM"))
 
-##     setGeneric("qqmath", function(x, data, ...)
-##                standardGeneric("qqmath"))
-
     setGeneric("tril", function(x, k = 0, ...)
                standardGeneric("tril"))
 
@@ -71,3 +66,9 @@ setGeneric("forceSymmetric",
 setGeneric("nnzero", function(x, na.counted = NA) standardGeneric("nnzero"),
 	   signature = "x")
 
+setGeneric("updown", function(update, C, L) standardGeneric("updown"))
+
+if(as.numeric(R.version$`svn rev`) < 60620)
+setGeneric("toeplitz", function(x, ...) standardGeneric("toeplitz"),
+           useAsDefault= function(x, ...) stats::toeplitz(x))
+## and an entry in ../man/sparseVector-class.Rd
