@@ -114,13 +114,18 @@ solve.dsC.dsC <- function(a,b, tol) {
 }
 setMethod("solve", signature(a = "dsCMatrix", b = "dsparseMatrix"),
 	  function(a, b, ...) {
-	      if (!is(b, "CsparseMatrix"))
-		  b <- as(b, "CsparseMatrix")
-	      if (is(b, "symmetricMatrix")) ## not supported (yet) by cholmod_spsolve
+	      cb <- getClassDef(class(b))
+	      if (!extends(cb, "CsparseMatrix"))
+		  cb <- getClassDef(class(b <- as(b, "CsparseMatrix")))
+	      if (extends(cb, "symmetricMatrix")) ## not supported (yet) by cholmod_spsolve
 		  b <- as(b, "dgCMatrix")
 	      solve.dsC.dsC(a,b)
 	  })
 
+setMethod("solve", signature(a = "dsCMatrix", b = "missing"),
+	  function(a, b, ...) solve(a, .trDiagonal(nrow(a), unitri=FALSE)))
+
+
 
 setMethod("chol", signature(x = "dsCMatrix"),
 	  function(x, pivot = FALSE, ...) .Call(dsCMatrix_chol, x, pivot),
@@ -137,12 +142,22 @@ setMethod("t", signature(x = "dsCMatrix"),
           function(x) .Call(Csparse_transpose, x, FALSE),
           valueClass = "dsCMatrix")
 
+### These two are very similar, the first one has the advantage to be applicable to 'Chx' directly:
+
 .diag.dsC <- function(x, Chx = Cholesky(x, LDL=TRUE), res.kind = "diag") {
     force(Chx)
-    stopifnot(is.integer(Chx@p), is.double(Chx@x))
+    if(!missing(Chx)) stopifnot(.isLDL(Chx), is.integer(Chx@p), is.double(Chx@x))
     .Call(diag_tC, Chx@p, Chx@x, Chx@perm, res.kind)
+    ##    ^^^^^^^ from ../src/Csparse.c
 }
 
+## here, we  *could* allow a 'mult = 0' factor :
+.CHM.LDL.D <- function(x, perm = TRUE, res.kind = "diag") {
+    .Call(dsCMatrix_LDL_D, x, perm, res.kind)
+    ##    ^^^^^^^^^^^^^^^^ from ../src/dsCMatrix.c
+}
+
+
 ## FIXME:  kind = "diagBack" is not yet implemented
 ##	would be much more efficient, but there's no CHOLMOD UI (?)
 ##
@@ -150,6 +165,8 @@ setMethod("t", signature(x = "dsCMatrix"),
 ##       for diag(), apply *inverse* permutation
 ##    	q <- p ; q[q] <- seq_along(q); q
 
+
+
 ldet1.dsC <- function(x, ...) .Call(CHMfactor_ldetL2, Cholesky(x, ...))
 ## these are slightly faster (ca. 3 to 4 %):
 ldet2.dsC <- function(x, ...) {
