@@ -11,29 +11,6 @@ setAs("nsparseMatrix", "dsparseMatrix", function(from) as(from, "dMatrix"))
 
 ###------- Work via  as(*, lgC) : ------------
 
-## For multiplication operations, sparseMatrix overrides other method
-## selections.  Coerce a ddensematrix argument to a nsparseMatrix.
-
-setMethod("%*%", signature(x = "nsparseMatrix", y = "ndenseMatrix"),
-	  function(x, y) x %*% as(y, "nsparseMatrix"))
-
-setMethod("%*%", signature(x = "ndenseMatrix", y = "nsparseMatrix"),
-	  function(x, y) as(x, "nsparseMatrix") %*% y)
-
-setMethod("crossprod", signature(x = "nsparseMatrix", y = "ndenseMatrix"),
-	  function(x, y = NULL) crossprod(x, as(y, "nsparseMatrix")))
-
-setMethod("crossprod", signature(x = "ndenseMatrix", y = "nsparseMatrix"),
-	  function(x, y = NULL) crossprod(as(x, "nsparseMatrix"), y))
-
-## and coerce nsparse* to lgC*
-setMethod("%*%", signature(x = "nsparseMatrix", y = "nsparseMatrix"),
-	  function(x, y) as(x, "ngCMatrix") %*% as(y, "ngCMatrix"))
-
-setMethod("crossprod", signature(x = "nsparseMatrix", y = "nsparseMatrix"),
-	  function(x, y = NULL)
-	  crossprod(as(x, "ngCMatrix"), as(y, "ngCMatrix")))
-
 setMethod("is.na", signature(x = "nsparseMatrix"), is.na_nsp)
 
 setMethod("all", signature(x = "nsparseMatrix"),
